import { Environment } from './models';
import { EdgeService } from './edge_service';
import { FeatureHubConfig } from './feature_hub_config';
import { InternalFeatureRepository } from './internal_feature_repository';
export interface PollingService {
    get frequency(): number;
    poll(): Promise<void>;
    stop(): void;
    attributeHeader(header: string): Promise<void>;
}
export declare type FeaturesFunction = (environments: Array<Environment>) => void;
export declare abstract class PollingBase implements PollingService {
    protected url: string;
    protected _frequency: number;
    protected _callback: FeaturesFunction;
    protected stopped: boolean;
    protected _header: string;
    protected _etag: string;
    constructor(url: string, frequency: number, callback: FeaturesFunction);
    attributeHeader(header: string): Promise<void>;
    stop(): void;
    get frequency(): number;
    abstract poll(): Promise<void>;
    parseCacheControl(cacheHeader: string | undefined): void;
    protected delayTimer(): Promise<void>;
}
export interface NodejsOptions {
    timeout?: number;
}
export interface BrowserOptions {
    timeout?: number;
}
export declare type PollingClientProvider = (options: BrowserOptions, url: string, frequency: number, callback: FeaturesFunction) => PollingBase;
export declare class FeatureHubPollingClient implements EdgeService {
    private _frequency;
    private _url;
    private _repository;
    private _pollingService;
    private _options;
    private _startable;
    private readonly _config;
    private _xHeader;
    private _pollPromiseResolve;
    private _pollPromiseReject;
    private _pollingStarted;
    static pollingClientProvider: PollingClientProvider;
    constructor(repository: InternalFeatureRepository, config: FeatureHubConfig, frequency: number, options?: BrowserOptions | NodejsOptions);
    private _initService;
    contextChange(header: string): Promise<void>;
    clientEvaluated(): boolean;
    requiresReplacementOnHeaderChange(): boolean;
    close(): void;
    poll(): Promise<void>;
    private stop;
    private _restartTimer;
    private _pollFunc;
    private response;
}
